#include "mcu.h"
#include "common_data.h"
#include "device_manager.h"
#include "messages.h"

#define TIMER_VALUE_BEFORE_SLEEP            (uint16_t)(7000 / SYSTICK_PERIOD_MS) // unit is 1ms

//-------------------------------------

static uint16_t Timer_SleepMode; // timer responsible for sleep mode
static DeviceStatus_t DeviceStatus_Snapshot;

//-------------------------------------

void DeviceManager_Init() {
    Timer_SleepMode = TIMER_VALUE_BEFORE_SLEEP;
    DeviceStatus.Device_AwaitingLongKeyPress = 0;
    DeviceStatus.Device_PowerSwitchOn = 0;

    DeviceStatus_Snapshot = DeviceStatus;
}

void DeviceManager_Process() {

    // handle brief key press
    if (MSG_GET_KEYPRESSED_BRIEF()) {
        if (DeviceStatus.Device_PowerSwitchOn) {
            CurrentChannel++;
            if (CurrentChannel > (uint8_t) 3) {
                CurrentChannel = 1;
            }
            HAL_SaveSettings();
        }
    }

    // handle long key press
    if (MSG_GET_KEYPRESSED_LONG()) {
        // toggle power state
        DeviceStatus.Device_PowerSwitchOn ^= 1;
    }

    // start waiting..
    if (MSG_GET_START_AWAITING_LONG_KEYPRESS()) {
        DeviceStatus.Device_AwaitingLongKeyPress = 1;
    }

    // stop waiting..
    if (MSG_GET_STOP_AWAITING_LONG_KEYPRESS()) {
        DeviceStatus.Device_AwaitingLongKeyPress = 0;
    }

    // handle sleep mode
    if (MSG_GET_KEYSTATE_CHANGED()) {
        // defer entering to sleep mode in the case of button activity
        Timer_SleepMode = TIMER_VALUE_BEFORE_SLEEP;
    }
    if (0 == --Timer_SleepMode) {
        HAL_SleepMode();
        // reinitialize sleep mode timer after exiting from sleep
        Timer_SleepMode = TIMER_VALUE_BEFORE_SLEEP;
    }

    // refresh port lines 
    HAL_RefreshOutput(DeviceStatus_Snapshot);
    DeviceStatus_Snapshot = DeviceStatus;
}